/**************************************************************************
 *
 * smoke_detector.c
 *
 * ADPD188BI -> slave address: 0x64
 *
**************************************************************************/

#include "smoke_detector.h"
#include "system.h"

#define ADPD_REG_CONFIG_NUM_REGS	29

ADPD_register_t const adpd_registers_val[ADPD_REG_CONFIG_NUM_REGS] =
{
	{ 0x10, 0x0001 },	//set operating mode to 'program'

	{ 0x00, 0x1000 },	// reset: 0x0000/ Number of available bytes to be read from FIFO (bit[15:8])
	{ 0x02, 0x0005 },	// reset: 0x0000/ enable gpio0 + set active low
	{ 0x06, 0x0F00 },	// reset: 0x0000/ FIFO length threshold (bit[13:8])
	{ 0x11, 0x3065 },	// reset: 0x1000/ Channel A,B, Average, 16 bit sum of channels
	{ 0x12, 0x0050 },	// reset: 0x0028/ sampling frequency= 100 Hz
	{ 0x14, 0x011D },	// reset: 0x0541/led1 pulse during time slot a/ led3 pulse during time slot b
	{ 0x15, 0x0220 },	// reset: 0x0600/averaging factor time slot a/b = 4
	{ 0x18, 0x1F00 },	// reset: 0x2000/ Time Slot A Channel 1 ADC offset
	{ 0x19, 0x3FFF },	// reset: 0x2000/ Time Slot A Channel 2 ADC offset
	{ 0x1A, 0x3FFF },	// reset: 0x2000/ Time Slot A Channel 3 ADC offset
	{ 0x1B, 0x3FFF },	// reset: 0x2000/ Time Slot A Channel 4 ADC offset
	{ 0x1E, 0x1F00 },	// reset: 0x2000/ Time Slot B Channel 1 ADC offset
	{ 0x1F, 0x3FFF },	// reset: 0x2000/ Time Slot B Channel 2 ADC offset
	{ 0x20, 0x3FFF },	// reset: 0x2000/ Time Slot B Channel 3 ADC offset
	{ 0x21, 0x3FFF },	// reset: 0x2000/ Time Slot B Channel 4 ADC offset
	{ 0x22, 0x3031 },	// reset: 0x3000/ setting led3
	{ 0x23, 0x3033 },	// reset: 0x3000/ setting led1
	{ 0x24, 0x3031 },	// reset: 0x3000/ setting led2
	{ 0x25, 0x6317 },	// reset: 0x630c/ led fine adjust -> led1:10111, led2: 01100(default), led3: 01100(default)
	{ 0x30, 0x0320 },	// reset: 0x0320/ led pulse width (microseconds) and offset width (microseconds) -> time slot a
	{ 0x31, 0x0810 },	// reset: 0x0818/ led pulse count and pulse period (microseconds) -> time slot a
	{ 0x35, 0x0320 },	// reset: 0x0320/ led pulse width (microseconds) and offset width (microseconds) -> time slot b
	{ 0x36, 0x0810 },	// reset: 0x0818/ led pulse count and pulse period (microseconds) -> time slot b
	{ 0x39, 0x22F0 },	// reset: 0x22fc/ afe width+offset time slot a
	{ 0x3B, 0x22F0 },	// reset: 0x22fc/ afe width+offset time slot b
	{ 0x3C, 0x31C6 },	// reset: 0x3006/voltage cathode:1,8 V/afe channel power down -> power down ch2,ch3,ch4 from bpf amp and integrator op amp
	{ 0x4B, 0x269C },	// reset: 0x2612/ clk32k normal operation, adjust
	{ 0x4D, 0x0082 },

	{ 0x10, 0x0000 },	//set operating mode to 'standby'
};


int ADPD_init(ALT_AVALON_I2C_DEV_t *dev){
	int status=0,i=0;
	unsigned int value = 0;
	unsigned int reg_addr = 0;
	unsigned char txbuffer[3];
	unsigned char rxbuffer[2];

	status=ADPD_write_reg16(dev, 0x0F, 0x0001);	// sw_reset

	for(i = 0; i < ADPD_REG_CONFIG_NUM_REGS; i++)
	{
		if(status != 0) break;

		reg_addr = adpd_registers_val[i].address;
		value = adpd_registers_val[i].value;
		txbuffer[0]	= reg_addr;
		txbuffer[1] = ((unsigned char)((value >> 8) & 0x00ff));
		txbuffer[2] = (unsigned char)(value & 0x00ff);
		status = alt_avalon_i2c_master_tx(dev, txbuffer, 3, ALT_AVALON_I2C_NO_INTERRUPTS);
	}
	return status;
	//-> after init, smoke detector is in standby mode. write 0x0002 to register 0x10 for normal operation mode
}

int ADPD_write_reg16(ALT_AVALON_I2C_DEV_t *dev, unsigned char reg_addr,alt_u16 data){
	unsigned char txbuffer[3];
	txbuffer[0]	= reg_addr;
	txbuffer[1] = ((unsigned char)((data >> 8) & 0x00ff));
	txbuffer[2] = (unsigned char)(data & 0x00ff);
	int status=alt_avalon_i2c_master_tx(dev, txbuffer, 3, ALT_AVALON_I2C_NO_INTERRUPTS);
	return status;
}

int ADPD_read_reg16(ALT_AVALON_I2C_DEV_t *dev, unsigned char reg_addr, alt_u8 *data){
	alt_u8 txbuffer[1];
	alt_u8 rxbuffer[2]={0x00,0x00};
	txbuffer[0] = reg_addr;
	int status=alt_avalon_i2c_master_tx_rx(dev, txbuffer, 1, rxbuffer, 2, ALT_AVALON_I2C_NO_INTERRUPTS);
	*data=rxbuffer[1];		// read data byte[15:8]
	*data++;
	*data=rxbuffer[0];		// read data byte[7:0]
	return status;
}

int ADPD_read_fifo(ALT_AVALON_I2C_DEV_t *dev, unsigned char reg_addr,int *slot_a,int *slot_b){
	unsigned char txbuffer[1];
	unsigned char rxbuffer[4]={0x00,0x00,0x00,0x00};
	txbuffer[0]	= reg_addr;
	int status=alt_avalon_i2c_master_tx_rx(dev, txbuffer, 1, rxbuffer, 4, ALT_AVALON_I2C_NO_INTERRUPTS);
	*slot_a=(rxbuffer[3])<<8;	// read slot_a byte[15:8]
	*slot_a+=rxbuffer[2];		// read slot_a byte[7:0]
	*slot_b=rxbuffer[1]<<8;		// read slot_b byte[15:8]
	*slot_b+=rxbuffer[0];		// read slot_b byte[7:0]
	return status;

}
